using System.Threading;
using MediatR;
using Todo.Core.Interfaces.Persistence;
using Todo.Core.Interfaces.User;

namespace Todo.Core.Application.Queries.Todos;

public record GetTodosQuery : IRequest<IEnumerable<TodoViewModel>>
{
    internal class Handler : IRequestHandler<GetTodosQuery, IEnumerable<TodoViewModel>>
    {
        private readonly ICurrentUserService _currentUserService;
        private readonly ITodoRepository _todoRepository;

        public Handler(ICurrentUserService currentUserService, ITodoRepository todoRepository)
        {
            _currentUserService = currentUserService;
            _todoRepository = todoRepository;
        }

        public async Task<IEnumerable<TodoViewModel>> Handle(
            GetTodosQuery request,
            CancellationToken cancellationToken)
        {
            var userId = _currentUserService.GetUserId();
            if (userId is null)
                throw new InvalidOperationException("Cannot get userId");

            var todos = await _todoRepository.GetTodosAsync(userId);

            return todos.Select(TodoViewModel.From);
        }
    }
}